.group-detail-card {
    margin-bottom: 8px;
    border: none;
}

.group-detail-card-body {
    padding-top: 0px;
}

.experiment-details-header {
    margin: inherit;
    display: flex;
    flex-direction: row;
    font-size: 22px;
    font-weight: 500;
    margin-bottom: 11px;
    color: #0d74af;
}

.experiment-comparison-header {
    margin: 36px 0px 6px 0px;
    font-size: 20px;
    font-weight: 500;
}

.group-preview-row {
    justify-content: start;
    margin: inherit;
    border-top: 2px #dfdfdf solid;
}

.group-preview-title {
    margin: auto 0px auto 4px;
    padding: 16px 0px;
}

.group-preview-title-link {
    margin: auto 0px auto 4px;
}

.group-preview-model-names {
    margin: auto 0px;
    padding-left: 48px;
}

.show-group-detail-collapsable-btn {
    border-style: none;
    background-color: rgba(0, 0, 0, 0);
    transform: rotate(180deg);
    margin-left: auto;
}

.hide-group-detail-collapsable-btn {
    border-style: none;
    background-color: rgba(0, 0, 0, 0);
    margin-left: auto;
}

.dash-table-container {
    /* make the action btns line up */
    margin-top: -40px;
}

.show-hide {
    box-shadow: none;
    box-sizing: border-box;
    font-weight: 400;
    font-family: Optimist, "Helvetica Neue", Helvetica, Arial, sans-serif;
    cursor: pointer;
    white-space: nowrap;
    color: #ffffff;
    background-color: #0D74AF;
    font-size: 14px;
    line-height: 2;
    border-width: initial;
    border-style: none;
    border-color: initial;
    border-image: initial;
    text-decoration: none;
    margin: 0px;
    outline: none;
    padding: 0px 16px;
    border-radius: 4px;
    margin-bottom: 6px;
}

.show-hide-menu .show-hide-menu-item, input {
    margin-top: 6px;
    margin-right: 6px;
}


.dash-spreadsheet-menu {
    justify-content: flex-end;
}

.dash-spreadsheet-container .dash-spreadsheet {
    margin-top: 8px;
}